/*
 *  This file is part of M-Index library: http://disa.fi.muni.cz/m-index/
 *
 *  M-Index library is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  M-Index library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with M-Index library.  If not, see <http://www.gnu.org/licenses/>.
 */
package mindex;

import java.util.Collection;
import messif.algorithms.AlgorithmMethodException;
import messif.objects.LocalAbstractObject;

/**
 * This is class is a simple extension of {@link MIndexPPCalculator}. It can calculate
 *  PP of given objects or a collection of objects (sequentially).
 * 
 * @author David Novak, Masaryk University, Brno, Czech Republic, novak.david@gmail.com
 */
public class SequentialPPCalculator extends MIndexPPCalculator {

    /** Class serial ID for serialization */
    private static final long serialVersionUID = 112090L;

    /**
     * Creates the PP calculator given M-Index properties
     * @param mIndexProperties properties with M-Index configuration
     * @throws InstantiationException if this part of M-Index cannot be initialized using this properties
     */
    public SequentialPPCalculator(MIndexProperties mIndexProperties) throws InstantiationException {
        super(mIndexProperties);
    }        
    
    // ***************       Init object - calculate obj-pivot distance + PP  *********************  //
    
    /**
     * Initialize the object in M-Index: calculate all distances, sort them, set pivot permutation (and set
     *   filter, if the filter should be stored by this M-Index).
     * @param object object to initialize
     */
    @Override
    public LocalAbstractObject initPP(LocalAbstractObject object) throws AlgorithmMethodException {
        initObject(object, isUsePivotFiltering());
        return object;
    }

    /**
     * Initialize the object in M-Index: calculate all distances, sort them, set pivot permutation (and set
     *   filter, if the filter should be stored by this M-Index).
     * @param object object to initialize
     * @return distances to pivots
     */
    @Override
    public float [] initPPGetDistances(LocalAbstractObject object) throws AlgorithmMethodException {
        return initObject(object, true);
    }
    
    /**
     * Initializes the PP of a collection of objects
     * @param objects the collection of objects to initialize the PP for
     */
    @Override
    public Collection<? extends LocalAbstractObject> initPP(Collection<? extends LocalAbstractObject> objects) throws AlgorithmMethodException {
        for (LocalAbstractObject localAbstractObject : objects) {
            initPP(localAbstractObject);
        }
        return objects;
    }
    
    
}
