/*
 *  This file is part of M-Index library: http://disa.fi.muni.cz/m-index/
 *
 *  M-Index library is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  M-Index library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with M-Index library.  If not, see <http://www.gnu.org/licenses/>.
 */
package mindex.distance;

import messif.algorithms.AlgorithmMethodException;
import messif.operations.RankingSingleQueryOperation;
import mindex.MetricIndex;


@Deprecated
public class WeightedSumDistCalculatorWrong extends QueryPivotDistDiffCalculator implements PartialQueryPPPDistanceCalculator {

    private float [] COEFS;
    private float [] COEF_SUMS;
    
    private float [] [] minValues;

    public WeightedSumDistCalculatorWrong(RankingSingleQueryOperation operation, MetricIndex mIndex) throws AlgorithmMethodException {
        super(operation.getQueryObject(), mIndex);
        initCoeficients(operation.getParameter(WeightedSumDistCalculator.WEIGHTED_SUM_COEF_PARAM, Float.class, WeightedSumDistCalculator.DEFAULT_COEF));
    }
    
    protected WeightedSumDistCalculatorWrong(short maxLevel, float [] queryPivotDistances, short [] queryPPP) {
        super(maxLevel, queryPivotDistances, queryPPP);
        initCoeficients(WeightedSumDistCalculator.DEFAULT_COEF);
    }
    
    private void initCoeficients(float coeficient) {
        minValues = new float[maxLevel + 1][maxLevel + 1];
        COEFS = new float[maxLevel + 1];
        COEF_SUMS = new float[maxLevel + 1];
        COEFS[0] = 1f;
        for (int i = 1; i < minValues.length; i++) {
            COEFS[i] = COEFS[i-1] * coeficient;
            COEF_SUMS[i] = COEF_SUMS[i-1] + COEFS[i-1];
            //minValues[i] += (queryPivotDistances[queryPPP[i - 1]] / (float) i);
            float minValueAtLevel = (queryPivotDistances[queryPP[i - 1]] * COEFS[i - 1]);
            for (int j = 0; j <= i; j++) {
                minValues[j][i] = minValues[j][i-1] + minValueAtLevel;
            }
        }
    }

    // **************   Distance calculations    ************************** //
    
    @Override
    public final float getQueryDistance(final short[] ppp) {
        return getDistance(ppp, 1, getMaxLevel(ppp.length));
    }

    @Override
    public float getPartialQueryDistance(final short[] ppp, final int fromLevel) {
        if (ppp.length == 0) {
            return 0f;
        }
        return getDistance(ppp, fromLevel, getMaxLevel(fromLevel - 1 + ppp.length));
    }

    private float getDistance(final short[] ppp, final int fromLevel, final int toLevel) {
        float sum = 0f;
        for (int i = 0; i <= toLevel - fromLevel; i++) {
            sum += (queryPivotDistances[ppp[i]] * COEFS[i + fromLevel - 1]);
        }
        return sum - minValues[fromLevel][toLevel];
    }

    @Override
    public float getPartialQueryDistance(short ppIndex, int fromLevel) {
        return (queryPivotDistances[ppIndex] * COEFS[fromLevel - 1]) - minValues[fromLevel][fromLevel];
    }
    
    
    public final float getCellDistanceEstimation(short[] ppp) {        
        float sum = 0f;
        for (int i = 0; i < getMaxLevel(ppp.length); i++) {
            sum += (queryPivotDistances[ppp[i]] * COEFS[i]);
        }
        
        return (ppp.length == 0) ? sum : sum / COEF_SUMS[getMaxLevel(ppp.length)];
    }

}
